#!/bin/sh
set -e
set -x

libtoolize --automake --copy
aclocal -I m4
autoheader
automake --add-missing --copy
autoconf
export CFLAGS="-g -O0"
export CXXFLAGS="$CFLAGS"
./configure --prefix='/usr' --enable-maintainer-mode "$@" --with-python=python3
